﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;
using CatanGame.Model;
using CatanGame.View;
using CatanGame.ViewModel;

namespace CatanGame
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App : Application
    {
        private MainWindow _mainWindow;
        private CatanViewModel _viewModel;
        private GameModel _model;
        private NewGameWindow _newGameWindow;
        private GameWindow _gameWindow;


        App()
        {
            Startup += new StartupEventHandler(App_Startup);
        }

        #region Application event handlers

        private void App_Startup(object sender, StartupEventArgs e)
        {
            _mainWindow = new MainWindow();
            _model = new GameModel();
            _newGameWindow = new NewGameWindow();
            _gameWindow = new GameWindow();

            //_model.NewGame(new List<String> { "Geza", "Bela", "Anna" });

            _viewModel = new CatanViewModel(_model);
            _viewModel.GameRolled += View_Rolled;
            _viewModel.UpgradeNode += ViewModel_UpgradeNode;
            _viewModel.PlaceRoad += ViewModel_PlaceRoad;
            _model.GameOver += new EventHandler<int>(Model_GameOver);

            //_newGameWindow.DataContext = _viewModel;

            _gameWindow.boardCanvas.ViewModel = _viewModel;
            //_gameWindow.DataContext = _viewModel;
            _gameWindow.exitButton.Click += exitButton_clicked;

            //_newGameWindow.DataContext = _viewModel;
            _mainWindow.DataContext = _viewModel;
            _viewModel.NewGameView += (o, args) => { _mainWindow.Navigate(_gameWindow); };
            _mainWindow.Navigate(_newGameWindow);
            _mainWindow.Show();
        }

        #endregion

        private void ViewModel_UpgradeNode(object sender, NodeCoordinates coords)
        {
            _model.UpgradeNode(coords, _model.CurrentPlayerId);
        }

        private void ViewModel_PlaceRoad(object sender, (int FirstNodeId, int SecondNodeId) pos)
        {
            _model.PlaceRoad(pos);
        }
        private void exitButton_clicked(object sender, RoutedEventArgs e)
        {
            _mainWindow.Close();
        }

        private void View_Rolled(object obj, RollEventArgs e)
        {
            _gameWindow.View_Rolled(e);
        }

        private void Model_GameOver(object obj, int id)
        {
            MessageBox.Show(_viewModel.Players[id].Name + " nyert!");
        }
    }
}
