﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CatanGame.Model
{
    public class GameModel
    {
        #region fields
        private int _decies1 = 2;
        private int _decies2 = 1;

        #endregion


        #region Events
        public event EventHandler<NodeCoordinates> NodeUpgraded;
        public event EventHandler<RoadPlacedEventArgs> RoadPlaced;
        public event EventHandler<RoadsEnabledEventArgs> RoadsEnabled;
        public event EventHandler<List<NodeCoordinates>> NodesEnabled;
        public event EventHandler GameStepped;
        public event EventHandler<RollEventArgs> GameRolled;
        public event EventHandler<int> GameOver;
        #endregion

        #region Private methods
        private NodeCoordinates GetCoordinates(int nodeId)
        {
            int r = nodeId / (Board.Size * 2);
            int q = (nodeId / 2) % Board.Size;
            Direction dir = (Direction)((nodeId) % 2);
            return new NodeCoordinates(r, q, dir);
        }

        private void ResendOptions()
        {
            SendEnabledRoads(CurrentPlayerId);
            SendEnabledNodes(CurrentPlayerId);
        }
        #endregion

        public void NewGame()
        {
            Board = new CatanBoard();
        }

        //dobokocka ertekeinek gettere
        public int Decies1 { get { return _decies1; } }
        public int Decies2 { get { return _decies2; } }

        public void BuyCard(int playerId)
        {
            Board.BuyCard(Board.Players[playerId]);

            if (Board.Players[CurrentPlayerId].Score > 4)
                GameOver?.Invoke(this, CurrentPlayerId);
        }

        public void SendEnabledRoads(int playerId)
        {
            RoadsEnabled?.Invoke(this, new RoadsEnabledEventArgs(Board.EnabledRoads(Board.Players[playerId])));
        }

        public void SendEnabledNodes(int playerId)
        {
            NodesEnabled?.Invoke(this, Board.EnableNodes(Board.Players[playerId]));
        }

        public void SendEnabledRoadsFromNode(int playerId, NodeCoordinates pos)
        {
            RoadsEnabled?.Invoke(this, new RoadsEnabledEventArgs(Board.EnabledRoadsFromNode(Board.Players[playerId], pos)));
        }

        public void PlaceRoad((int FirstNodeId, int SecondNodeId) pos)
        {
            //System.Windows.MessageBox.Show(GetCoordinates(pos.FirstNodeId) + ", " + GetCoordinates(pos.SecondNodeId));

            int currPlayer = CurrentPlayerId;
            NodeCoordinates firstNodeCoords = GetCoordinates(pos.FirstNodeId);
            NodeCoordinates secondNodeCoords = GetCoordinates(pos.SecondNodeId);

            if (!Board.PlaceRoad(new RoadCoordinates(firstNodeCoords, secondNodeCoords), Board.Players[CurrentPlayerId]))
                return;

            RoadPlaced?.Invoke(this, new RoadPlacedEventArgs() {
                FirstNode = firstNodeCoords, 
                SecondNode = secondNodeCoords,
                OwnerId = currPlayer,
            });

            if (Board.InitialTurn)
                SendEnabledNodes(CurrentPlayerId);

            if (Board.Players[CurrentPlayerId].Score > 4)
                GameOver?.Invoke(this, CurrentPlayerId);
        }

        public void UpgradeNode(NodeCoordinates node, int playerId)
        {
            bool canPlace;
            if(Board.Nodes[node].Building.Type == HouseType.NoBuilding)
                canPlace = Board.PlaceSettlement(node, Board.Players[playerId]);
            else
                canPlace = Board.UpgradeToCity(node, Board.Players[playerId]);

            if (!canPlace)
                return;

            NodeUpgraded?.Invoke(this, node);

            if (Board.InitialTurn)
                SendEnabledRoadsFromNode(playerId, node);

            if (Board.Players[CurrentPlayerId].Score > 4)
                GameOver?.Invoke(this, CurrentPlayerId);
        }

        public void StepGame()
        {
            Board.StepGame();
            GameStepped?.Invoke(this, EventArgs.Empty);
        }

        public bool Trade(ResourceType type1, ResourceType type2, int cnt = 1)
        {
            return Board.Trade(type1, type2, cnt);
        }

        public (int, int) Roll()
        {
            (int, int) rollResult = Board.Roll();
            SendResources(rollResult.Item1 + rollResult.Item2);
            (_decies1, _decies2) = rollResult;
            GameRolled?.Invoke(this, new RollEventArgs( _decies1, _decies2));
            return rollResult; //itt void lehetne

        }


        private void SendResources(int num)
        {
            if (num != 7)
                Board.SendResources(num);
        }

        public CatanBoard Board { get; set; }
        public int CurrentPlayerId { get => Board.CurrentPlayerId; }
    }
}
