﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CatanGame.Model
{
    public class NodeMatrix
    {
        private Node[,,] _nodes;

        public int BoardSize { get; private set; }

        public NodeMatrix(int boardSize)
        {
            BoardSize = boardSize;
            _nodes = new Node[BoardSize, BoardSize, 2];
            for (int q = 0; q < BoardSize; ++q)
            {
                for (int r = 0; r < BoardSize; ++r)
                {

                    if (IsInnerBoard(r, q))
                    {
                        _nodes[r, q, (int)Direction.Bottom] = new();
                        _nodes[r, q, (int)Direction.Bottom].Building = new(null);
                        _nodes[r, q, (int)Direction.Top] = new();
                        _nodes[r, q, (int)Direction.Top].Building = new(null);
                    }
                    else if (Math.Abs(q + r - BoardSize + 1) < 1 + BoardSize / 2)
                    {
                        if (r > BoardSize / 2)
                        {
                            _nodes[r, q, (int)Direction.Top] = new();
                            _nodes[r, q, (int)Direction.Top].Building = new(null);
                        }
                        else if (r < BoardSize / 2)
                        {
                            _nodes[r, q, (int)Direction.Bottom] = new();
                            _nodes[r, q, (int)Direction.Bottom].Building = new(null);
                        }
                    }
                }
            }
        }

        private bool IsInnerBoard(int r, int q)
        {
            return Math.Abs(q - BoardSize / 2 + r - BoardSize / 2) < 1 + (BoardSize - 2) / 2 && q != 0 && q != BoardSize - 1 && r != 0 && r != BoardSize - 1;
        }

        public bool NodeExists(NodeCoordinates coords)
        {
            return coords.R >= 0 
                && coords.Q >= 0 
                && coords.R < BoardSize 
                && coords.Q < BoardSize 
                && this[coords] is not null;
        }

        // private on purpose

        public Node this[NodeCoordinates coords]
        {
            get => GetNode(coords);
            set => SetNode(coords, value);
        }

        public Node this[int r, int q, Direction dir]
        {
            get => GetNode(new NodeCoordinates(r, q, dir));
            set => SetNode(new NodeCoordinates(r, q, dir), value);
        }

        private Node GetNode(NodeCoordinates coords)
        {
            return GetNode(coords.R, coords.Q, coords.Dir);
        }

        private Node GetNode(int r, int q, Direction dir)
        {
            return _nodes[r, q, (int) dir];
        }

        private void SetNode(NodeCoordinates coords, Node value)
        {
            _nodes[coords.R, coords.Q, (int)coords.Dir] = value;
        }
    }
}
