﻿using System;
using System.ComponentModel;
using System.Runtime.CompilerServices;

namespace CatanGame.ViewModel
{
    // Code taken from class resources
    public abstract class ViewModelBase : INotifyPropertyChanged
    {
        /// <summary>
        /// Nézetmodell ősosztály példányosítása.
        /// </summary>
        protected ViewModelBase() { }

        /// <summary>
        /// Tulajdonság változásának eseménye.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;


        /// <summary>
        /// Tulajdonságot változtató metódus
        /// </summary>
        /// <typeparam name="T">A tulajdonság típusa</typeparam>
        /// <param name="property">A tulajdonság értékét képviselő referencia</param>
        /// <param name="value">Az új érték</param>
        /// <param name="propertyName">Tulajdonság neve</param>
        public void ChangeProperty<T>(ref T property, T value, [CallerMemberName] String propertyName = null) where T : IEquatable<T>
        {
            if (property is not null && property.Equals(value))
                return;

            property = value;
            OnPropertyChanged(propertyName);
        }

        /// <summary>
        /// Tulajdonság változása ellenőrzéssel.
        /// </summary>
        /// <param name="propertyName">Tulajdonság neve.</param>
        protected virtual void OnPropertyChanged([CallerMemberName] String propertyName = null)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}
